; ### VIChaos ###
; Port of the Spectrum game Chaos for VIC-20.
; For 16k+.

; Based on the GBA port sources (chaos-0.82). Thanks quirky!

; $1000 - $11ff : vmatrix
; $1201 - $12xx : basic start
; $12xx - $2fff : ml entry, main code, RO data
; $3000 - $3fff : RW data, phase 0 init code

!ct "text_convtab.bin"

; # const

!src "const.asm"

; # addr

!src "addr.asm"


; ###############

; # code entry

* = 4621
ml_entry:
    jmp init_phase0


; # interrupt handlers

!zn INT_Handler {
irq_handler:
    lda $9124   ; ACK IRQ
!if DEBUG_IRQ_LEN > 0 {
    lda #$11
    sta $900f
}
    jsr sfx_tick

    ldy irq_downcount
    beq +
    dey
    sty irq_downcount
+
    ; read the joystick
    jsr input_joy_tick
    ; read the keyboard
    jsr $ff9f

irq_exit:
; DEBUG
!if DEBUG_IRQ_LEN > 0 {
    lda #$08
    sta $900f
}

brk_handler:
    pla
    tay
    pla
    tax
    pla
    rti

nmi_handler:
    pha
    lda k_lstx
    cmp #$18    ; run/stop
    bne +
    ; restart via main_entry
    ldx #<stack_top
    txs
    lda #>main_entry
    pha
    lda #<main_entry
    pha
    lda #0
    pha ; flags
    pha ; for the pla below
+   lda $9111   ; ACK NMI restore
    pla
    rti
}   ; !zn INT_Handler


; ###############

; # main data

; game phase handler pointers
; out:
;  C = 0 if continue
;  C = 1 if end
;  A = 0..7 winner | <0 quit
; phase 0: game menu: select spell, examine board
; phase 1: cast spell
; phase 2: movement
tbl_game_phaseh:
!by >game_menu
!by >cast_phase
!by >movement_phase
!by 0
tbl_game_phase:
!by <game_menu
!by <cast_phase
!by <movement_phase
;!by 0  ; not needed


; # main code

!zn Main_code {
main_entry:
    ; stop sound
    jsr sfx_stop

    ; create_default_wizs
    ldy #NUM_PLAYERS_MAX - 1
    lda #4  ; cpu level
-   ldx wizard_offs,y
    sta player_data + PDATA_OFFS_PTYPE,x
    dey
    bne -
    lda #0  ; human
    sta player_data + PDATA_OFFS_PTYPE

main_loop:
    jsr show_splash

    ; reset_arena_tables
    ldx #AREA_SIZE
    lda #0
-   sta+2 arena_above - 1,x
    sta arena_aflag - 1,x
    sta arena_below - 1,x
    sta arena_bflag - 1,x
    sta arena_dead - 1,x
    dex
    bne -

!if 0 {
    ; los testing
    lda #1
    sta color
    jsr set_arena_border_color
    jsr arena_draw
-   lda #0
    jsr arena_view
    sta wcoord
    jsr arena_view_keep_coord
    lda wcoord
    lda #3
    sta current_spell
    jsr spell_animation
;    ldx #COLOR_YELLOW
;    jsr los_blocked
    jmp -
}

    jsr create_players
    jsr print_clear_status
    jsr arena_draw

    lda #0
    sta phase_num
    sta world_chaos
    sta round_count

game_loop_next_round:
    inc round_count
game_loop:
    ldy phase_num
    lda tbl_game_phaseh,y
    bne ++
    ; last phase
    sta phase_num
    beq game_loop_next_round

++  sta phaseptrh
    lda tbl_game_phase,y
    sta phaseptr

    ; flag_phase <- phase_num << 6
    tya
    clc
    ror
    ror
    ror
    sta flag_phase

    ; only phase 2 has prepare (and b7 of flag_phase set)
    bpl ++
    jsr movement_prepare
++

    ldy #0
game_phase_loop:
    sty player_i
    cpy player_num
    bne +
    inc phase_num
    bne game_loop   ; bra
+
    ; check if players left
    jsr game_check_win_and_cpu
    bcs game_over

    ; check if dead
    ldy player_i
    lda wizard_offs,y
    sta player_offs
    tax
    lda player_data + PDATA_OFFS_FLAGS1,x
    and #PDATA_MASK_FLAGS1_DEAD
    beq +
    iny
    jmp game_phase_loop
+
    ; set player vars
    ldy #0
    lda player_data + PDATA_OFFS_PTYPE,x
    beq +
    ; phase 0 is skipped by CPU
    lda phase_num
    beq game_phase_loop_next_player
    ldy #$80
+   sty flag_player_cpu
    bne +
    jsr flush_input ; flush input (only) for human players
+

    ; color border
    ldy player_i
    lda wizard_color,y
    sta color
    jsr set_arena_border_color

phaseptr = * + 1
phaseptrh = * + 2
    jsr game_menu
    bcs game_over

game_phase_loop_next_player:
    ldy player_i
    iny
    jmp game_phase_loop


.text_winner:
!tx "The winner is", 0
.text_all_dead:
   ; 01234567890123
!tx "Everyone died.", 0

; A = 0..7 winner | 8 draw | 9 all dead | <0 quit
game_over:
    bpl +
    jmp main_loop
+
    ; winner screen
    sta player_i
    jsr print_clear_all
    ldx #<(vmatrix + POS_AREA + COLS * 3 + 1)
    ldy #>(vmatrix + POS_AREA + COLS * 3 + 1)
    jsr print_setup_dst_xy
    ldx #COLS * 2 + 3
    ldy player_i
    cpy #9  ; all dead
    beq game_over_all_dead
    jsr print_wizard_name_offs_x
    ldx #<.text_winner
    stx zptxt
    ldy #>.text_winner
    sty zptxth
    jsr print_packed_dst_ready
-   inc zptxt
    lda zptxt
    lsr
    lsr
    lsr
    lsr
    and #$7
    jsr set_arena_border_color
    jsr get_input
    beq -
    jmp main_loop

game_over_all_dead:
    lda #COLOR_WHITE
    sta color
    ldx #<.text_all_dead
    stx zptxt
    ldy #>.text_all_dead
    sty zptxth
    jsr print_packed_dst_ready
    lda #0
    jsr set_arena_border_color
-   jsr get_input
    beq -
    jmp main_loop

game_check_win:
    lda #9  ; assume all dead
    sta zpdst   ; alive player_i | 8 draw | 9 all dead
    lda #0
    sta zptmp   ; num alive
    sta zpdsth  ; num alive CPU
    tay
-   ldx wizard_offs,y
    lda player_data + PDATA_OFFS_FLAGS1,x
    and #PDATA_MASK_FLAGS1_DEAD
    bne +
    inc zptmp
    sty zpdst   ; alive player_i
    lda player_data + PDATA_OFFS_PTYPE,x
    beq +
    inc zpdsth
+   iny
    cpy player_num
    bne -
    lda zptmp
    cmp #2
    bcc +
    ; 2 or more alive
    clc
    rts
+   ; less than 2 alive
    lda zpdst   ; alive player_i | 9 all dead
    sec
    rts


.text_cpucont:
!tx "Humans dead. ", TTOK_CONTINUE, "?",0

game_check_win_and_cpu:
    jsr game_check_win
    bcc +
    ; only less than 2 alive, winner found
    rts
+   ; 2 or more alive, a = num alive
    cmp zpdsth  ; num alive CPU
    bne ++
    jsr get_input
    beq ++
    ; some input given, ask if to continue
    jsr print_clear_status
    lda #COLOR_WHITE
    sta color
    ldx #<.text_cpucont
    ldy #>.text_cpucont
    stx zptxt
    sty zptxth
    jsr print_setup_dst_statusline
    jsr print_packed_dst_ready
    ldx #<text_noyes
    ldy #>text_noyes
    jsr statusmenu
    jsr print_clear_status
    cpy #0
    beq .game_cpu_stop
++  clc
    rts
.game_cpu_stop:
    ; quit
    lda #$ff
    sec
    rts

} ; !zn Main_code


; # subroutines and data

Start_debug = *
!src "debug.asm"
LEN_debug = * - Start_debug
Start_arena = *
!src "arena.asm"
LEN_arena = * - Start_arena
Start_casting = *
!src "casting.asm"
LEN_casting = * - Start_casting
Start_computer = *
!src "computer.asm"
LEN_computer = * - Start_computer
Start_creature = *
!src "creature.asm"
LEN_creature = * - Start_creature
Start_examine = *
!src "examine.asm"
LEN_examine = * - Start_examine
Start_gamemenu = *
!src "gamemenu.asm"
LEN_gamemenu = * - Start_gamemenu
Start_gfx = *
!src "gfx.asm"
LEN_gfx = * - Start_gfx
Start_input = *
!src "input.asm"
LEN_input = * - Start_input
Start_magic = *
!src "magic.asm"
LEN_magic = * - Start_magic
Start_movement = *
!src "movement.asm"
LEN_movement = * - Start_movement
Start_players = *
!src "players.asm"
LEN_players = * - Start_players
Start_random = *
!src "random.asm"
LEN_random = * - Start_random
Start_sound = *
!src "sound.asm"
LEN_sound = * - Start_sound
Start_spells = *
!src "spells.asm"
LEN_spells = * - Start_spells
Start_splash = *
!src "splash.asm"
LEN_splash = * - Start_splash
Start_statusmenu = *
!src "statusmenu.asm"
LEN_statusmenu = * - Start_statusmenu
Start_text = *
!src "text.asm"
LEN_text = * - Start_text
Start_util = *
!src "util.asm"
LEN_util = * - Start_util
Start_wizards = *
!src "wizards.asm"
LEN_wizards = * - Start_wizards

Code_end = *

; ###############

!if Code_end > Variable_base {
!error "Code over variables: ", Code_end - Variable_base
}

LEN_space_left = Variable_base - Code_end

* = Variable_base

; ###############

Start_phase0 = *
!src "phase0.asm"
LEN_phase0 = * - Start_phase0

!if * > RAM_16k_end {
!error "16k is not enough for p0: ", * - RAM_16k_end
}
