; input routines

!ifdef INCLUDE_INPUT !eof
!if INCLUDE_GUARD > 0 {
INCLUDE_INPUT = 1
}

!zn Input {

INPUT_NONE = 0
INPUT_U = 1
INPUT_D = 2
INPUT_L = 3
INPUT_R = 4
INPUT_UL = 5
INPUT_UR = 6
INPUT_DL = 7
INPUT_DR = 8
INPUT_SELECT = 9
INPUT_DEBUG = 10

.JOY_FIRST_FRAMES = 20
.JOY_REPEAT_FRAMES = 4
.JOY_BOUNCE_FRAMES = 3

input_joy_tick:
    ; get current state %000frldu
    lda $9111
    lsr
    tax
    and #$10    ; f
    sta input_joy_tick_smc
    txa
    lsr
    and #$07    ; udl
input_joy_tick_smc = * + 1
    ora #$00
    ldx #$00
    stx $9122
    bit $9120   ; b7 = r
    bpl +
    ora #$08    ; r
+   ldx #$ff
    stx $9122
    eor #%00011111
    ; compare to previous
    cmp joy_prev
    beq +
    ; changed
    sta joy_prev
    ; clear current value
    lda #INPUT_NONE
    sta joy_value
    ; debounce
    lda #.JOY_BOUNCE_FRAMES
    sta joy_count
    lda #1  ; debounce
    sta joy_repeat
    rts
+   ; check if counter is done
    dec joy_count
    beq +
    rts
+   ; decode joystick input
    ldx #INPUT_NONE
    cmp #0  ; no input
    beq input_joy_tick_repeat
    cmp #%00010000  ; fire
    bcc ++
    ldx #INPUT_SELECT
input_joy_tick_repeat:
    stx joy_value
    ldx #.JOY_REPEAT_FRAMES
    dec joy_repeat
    bmi +
    ldx #.JOY_FIRST_FRAMES    ; first repeat
+   lda #$ff    ; more repeats
    sta joy_repeat
    stx joy_count
    rts
++  ; one of the directions
    inx ; INPUT_U
    lsr
    bcc ++
    ; up, check diagonal
    lsr ; throw away d
    beq input_joy_tick_repeat
    ldx #INPUT_UL
    lsr ; l
    bcs input_joy_tick_repeat
    inx ; was UR
    bne input_joy_tick_repeat ; bra
++  inx ; INPUT_D
    lsr
    bcc ++
    ; down, check diagonal
    beq input_joy_tick_repeat
    ldx #INPUT_DL
    lsr ; l
    bcs input_joy_tick_repeat
    inx ; was DR
    bne input_joy_tick_repeat ; bra
++  inx ; INPUT_L
    lsr ; l
    bcs input_joy_tick_repeat
    inx ; INPUT_R
    bpl input_joy_tick_repeat ; bra ; it must be R


.k_getinkbd = $f1f9 ;$ffe4

get_input_wait_irq:
    lda #1
    sta irq_downcount
-   lda irq_downcount
    bne -
    ; FALL THROUGH

get_input:
!if DEBUG_JOY > 0 {
    ldy #0
    lda joy_prev
    jsr print_hex
    ldy #3
    lda joy_count
    jsr print_hex
    ldy #6
    lda joy_value
    jsr print_hex
    ldy #9
    lda joy_repeat
    jsr print_hex
}
    lda joy_value
    beq +
    pha
    ldx #0
    stx joy_value
    pla ; get Z flag back
    rts
+   jsr .k_getinkbd
    ldx #0
-   cmp .input_tbl_in,x
    beq .get_input_found
    bcc .get_input_not_found
    inx
    bne -   ; bra
.get_input_not_found:
    lda #INPUT_NONE
    rts
.get_input_found:
    lda .input_tbl_out,x
    rts

flush_input:
-   jsr get_input
    bne -
    rts

.input_tbl_in:
; NOTE these are sorted
!by $00
!if DEBUG_KEY > 0 {
!by $03 ; r/s
}
;   rtrn a    c    d    e    q    s    w    x    z
!by $0d, $41, $43, $44, $45, $51, $53, $57, $58, $5a

.input_tbl_out:
!by INPUT_NONE
!if DEBUG_KEY > 0 {
!by INPUT_DEBUG
}
!by INPUT_SELECT
!by INPUT_L
!by INPUT_DR
!by INPUT_R
!by INPUT_UR
!by INPUT_UL
!by INPUT_D
!by INPUT_U
!by INPUT_D
!by INPUT_DL

; direction to arena offset
tbl_input_dir:
; assumes order:
;      U,       D,      L,  R, UL,          UR,          DL,         DR,         select
!by 0, -AREA_W, AREA_W, -1, 1, -AREA_W - 1, -AREA_W + 1, AREA_W - 1, AREA_W + 1, 0
!if DEBUG_KEY > 0 {
!by $0  ; debug
}


; out:
; a = screencode letter | $00 = invalid | $ff = cancel | $fe = done | $fd = delete
; (any joystick input should return $ff)
get_input_text:
-   jsr .k_getinkbd
    beq -
!if DEBUG_EDITNAME > 0 {
    ldy #0
    pha
    jsr print_hex
    pla
}
    cmp #$0d    ; rtrn
    bne +
    lda #$fe
    rts
+   cmp #$14    ; backspace
    bne +
    lda #$fd
    rts
+   cmp #$03    ; run/stop
    bne +
    lda #$ff
    rts
+   cmp #$20    ;  (space)
    bcc -
    cmp #$3f + 1    ; ? + 1
    bcs +
    rts
+
    cmp #$41    ; a
    bcc ++
    cmp #$5a + 1    ; z + 1
    bcs ++
    ; clc
    sbc #$41 - ('a' + 1)    ; + 1 for clc
    rts
++
    cmp #$c1    ; A
    bcc ++
    cmp #$da + 1    ; Z + 1
    bcs ++
    ; clc
    sbc #$c1 - ('A' + 1)    ; + 1 for clc
    rts
++
    jmp -

} ; !zn Input
