; spell implementations

!ifdef INCLUDE_MAGIC !eof
!if INCLUDE_GUARD > 0 {
INCLUDE_MAGIC = 1
}

!zn Magic {

; cast_* handlers
; in:
; player_i = current_player
; current_spell = spell to cast
; wcoord = caster coord
; acoord = target coord
; print_dst is set to area
; inout:
; temp_cast_amount = 0 when first called and 0 when finishing

cast_creature:
    lda #1
    sta temp_cast_amount
    bit flag_player_cpu
    bpl +
    jmp ai_cast_creature
+
    lda #30
    ; FALL THROUGH

; a = delay
cast_check_player_ok:
    sta .cast_check_player_ok_delay
    jsr player_cast_ok
    bcs +
    rts
+   ; target ok
    jsr do_wall_cast    ; works for creatures too
    lda temp_cast_amount
    bne +
    jsr cast_print_success_status
+
.cast_check_player_ok_delay = * + 1
    lda #30
    jsr delay
    rts


do_wall_cast:
    jsr spell_animation
    bit temp_success_flag
    bpl +
    ldx acoord
    jsr creature_spell_succeeds
    ldy acoord
    jsr arena_draw_square
    dec temp_cast_amount
    bpl ++
+   lda #0
    sta temp_cast_amount
++  rts


cast_trees_castles:
    lda current_spell
    ldy temp_cast_amount
    bne +
    ldy #1
    cmp #SPELL_MAGIC_CASTLE
    bcs +
    ldy #8
+   sty temp_cast_amount

    bit flag_player_cpu
    bpl +++
    ; CPU
    cmp #SPELL_MAGIC_CASTLE
    bcs ++
    ; shadow wood or magic wood - uses the same routine
    ldy #8
-   sty temp_cast_amount ; FIXME redundant?
    jsr cast_print_spell_name
    lda #80
    jsr delay
    jmp auto_cast_trees_castles
++  ; CPU, castle. check if spell is worth casting
    ldx wcoord
    lda arena_above,x
    cmp #SPELL_MAGIC_CASTLE
    bcc +
    cmp #SPELL_DARK_CITADEL + 1
    bcs ++
+   ldy #1
    bne -   ; bra
++  ; not worth casting
    lda #0
    sta target_square_found
    rts

+++ ; human
    cmp #SPELL_MAGIC_WOOD
    bne +
    ldy #8
    sty temp_cast_amount ; FIXME redundant?
    jmp auto_cast_trees_castles
+   ; do same checks as for creatures
    lda #10
    jmp cast_check_player_ok


cast_wall:
    ldy temp_cast_amount
    bne +
    ldy #4
    sty temp_cast_amount
+
    bit flag_player_cpu
    bpl +++
    ; CPU
    jsr ai_cast_wall
    bit target_square_found
    bpl +
    jsr cast_print_success_status
    lda #30
    jsr delay
+   rts

+++ ; human
    ; do same checks as for creatures
    lda #30
    jmp cast_check_player_ok
    rts


cast_magic_missile:
    lda #1
    sta temp_cast_amount
    bit flag_player_cpu
    bpl +
    jmp ai_cast_magic_missile
+
    ; check this is a valid square for human casting
    ldy acoord
    lda arena_above,y
    bne +
-   rts
+
    jsr is_spell_in_range   ; in range?
    bcs -
    lda wcoord
    ldx #0  ; check LOS
    jsr los_blocked
    bcc -
    ; got to here? well cast the spell!
    jsr do_magic_missile
    jmp cast_finish

do_magic_missile:
    ldy #SND_ELECTRO
    ldx #COLOR_YELLOW
    lda current_spell
    cmp #SPELL_LIGHTNING
    beq +
    ldy #SND_SPELLSUCCESS
    ldx #COLOR_WHITE
+   sty .do_magic_missile_sound
    stx .do_magic_missile_color
    lda #SND_BEAM
    jsr sound_play
    lda wcoord
    jsr los_blocked
.do_magic_missile_sound = * + 1
    lda #0
    jsr sound_play
.do_magic_missile_color = * + 1
    ldx #0
    jsr splat_animation

    ldy acoord
    lda arena_above,y
    cmp #SPELL_MAGIC_FIRE
    bne +
-   ; no effect
    rts
+
    cmp #SPELL_MAGIC_CASTLE
    bcc +
    cmp #SPELL_WALL + 1
    bcc -
+

    ; get the defence of the creature
    jsr creature_get_def
    sta zptmp   ; def
    jsr get_random_10
    clc
    adc zptmp   ; def
    sta zptmp   ; def

    ; get attack
    jsr get_random_10
    clc
    adc #3
    ldx current_spell
    cmp #SPELL_LIGHTNING
    bne +
    clc
    adc #3
+
    cmp zptmp   ; def
!if DEBUG_MAGIC_ALWAYS_WORKS = 0 {
    bcs +
    ; attack < defence
    rts
+
}
    ldx .do_magic_missile_color
    jsr pop_animation

    ldy acoord
    lda arena_below,y
    bne .do_magic_missile_got_below
    ; nothing below
    lda arena_above,y
    cmp #WIZARD_INDEX
    bcc +
    jsr kill_wizard
    rts
+   ; remove the creature
    lda #0
    sta arena_above,y
    lda arena_bflag,y
    bne +
    rts
+   sta arena_above,y
    lda #0
    sta arena_bflag,y
    lda arena_aflag,y
    ora #DEAD_MASK_IS_DEAD
    sta arena_dead,y
    rts

.do_magic_missile_got_below:
    ; something below
    sta zptmp   ; *arena_below
    lda #0
    sta arena_below,y
    lda arena_above,y
    cmp #SPELL_GOOEY_BLOB
    bne +
    lda arena_bflag,y
    sta arena_aflag,y
    jmp ++
+   lda zptmp   ; *arena_below
    cmp #WIZARD_INDEX
    bcc ++
    ; sec
    sbc #WIZARD_INDEX
    sta arena_aflag,y
++  lda zptmp   ; *arena_below
    sta arena_above,y
    lda #0
    sta arena_bflag,y
    rts

cast_finish:
    ldy acoord
    jsr arena_draw_square
    lda #0
    sta temp_cast_amount
    lda #30
    jsr delay
    rts


cast_subversion:
    lda #1
    sta temp_cast_amount
    bit flag_player_cpu
    bpl ++
    jsr ai_cast_subversion
    bit target_square_found
    bmi do_subversion
-   rts

++  ; human
    ; check that a proper creature is in the square
    ; a creature is ridden by a wizard is rejected
    ldy acoord
    lda arena_above,y
    beq -
    cmp #SPELL_GOOEY_BLOB
    bcs -
    lda arena_below,y
    bne -

    jsr is_spell_in_range   ; in range?
    bcs -
    lda wcoord
    ldx #0  ; check LOS
    jsr los_blocked
    bcc -
    ; got to here then spell is valid
    ; FALL THROUGH

do_subversion:
    lda #0
    sta temp_cast_amount
    sta temp_success_flag
    bit flag_player_cpu
    bpl ++
    jsr cast_print_spell_name
    lda #80
    jsr delay
++  jsr spell_animation
    ldy acoord
    lda arena_aflag,y
    and #AFLAG_MASK_ILLUSION
    bne ++
    ; not an illusion
    lda arena_above,y
    jsr creature_get_magic_res
    clc
    adc #1
    sta zptmp   ; mr + 1
    jsr get_random_10
    cmp zptmp   ; mr + 1
!if DEBUG_MAGIC_ALWAYS_WORKS = 0 {
    bcc ++  ; r < mr + 1
}
    ldy acoord
    lda arena_aflag,y
    and #!AFLAG_MASK_OWNER
    ora player_i
    sta arena_aflag,y
    dec temp_success_flag
++
cast_finish_print_square_success:
    jsr print_setup_dst_area
    ldy acoord
    jsr arena_draw_square
cast_finish_print_success:
    jsr cast_print_success_status
    lda #30
    jsr delay
    rts


cast_chaos_law:
    bit flag_player_cpu
    bpl ++
    jsr cast_print_spell_name
    lda #80
    jsr delay
++  ; check success (for CPU) and update world chaos (again for player)
    jsr set_spell_success
    lda #$80
    sta target_square_found
    lda #0
    sta temp_cast_amount
    jmp cast_finish_print_square_success


cast_justice:
    lda temp_cast_amount
    bne ++
    lda #1
    ldy current_spell
    cpy #SPELL_DARK_POWER
    bcc +
    lda #3
+   sta temp_cast_amount
++
    bit flag_player_cpu
    bpl ++
    ; this will always be chosen, but might not actually be any good
    jmp ai_cast_justice
++  ; human
    ; jsr clear_message
    bit temp_success_flag
    bmi +
    lda #0
    sta temp_cast_amount
-   rts
+   ; ok, got here then check spell is OK to cast here
    ldy acoord
    lda arena_above,y
    beq -
    ; only allow on proper creatures and wizards
    cmp #WIZARD_INDEX
    bcs +
    cmp #SPELL_GOOEY_BLOB
    bcs -
+   jsr do_justice_cast
    dec temp_cast_amount
    rts


do_justice_cast:
    ; do sound fx, flashing graphics
    lda #SND_JUSTICE
    jsr sound_play
    ldy acoord
    lda arena_aflag,y
    and #AFLAG_MASK_OWNER
    tay
    ldx wizard_color,y
    ldy #60
    jsr flash_anim

    ; get magic resistance of creature
    ldy acoord
    lda arena_above,y
    jsr creature_get_magic_res
    clc
    adc #1
    sta zptmp   ; mr + 1
    jsr get_random_10
    cmp zptmp   ; mr + 1
!if DEBUG_MAGIC_ALWAYS_WORKS = 0 {
    bcc +++  ; r < mr + 1
}
    ldx #COLOR_WHITE
    jsr pop_animation
    ldy acoord
    lda arena_above,y
    cmp #WIZARD_INDEX
    bcc +
    ; wizard - destroy all his creations!
    jsr destroy_all_creatures_id
    rts
+   ; single creature only
    lda #0
    sta arena_above,y
    lda arena_below,y
    beq +
    sta arena_above,y
    lda #0
    sta arena_below,y
    beq +++ ; bra
+
    lda arena_bflag,y
    beq +++
    sta arena_above,y
    lda #0
    sta arena_bflag,y
    lda arena_aflag,y
    ora #DEAD_MASK_IS_DEAD
    sta arena_dead,y
+++
    ldy acoord
    jsr arena_draw_square
    rts


cast_raise_dead:
    lda temp_cast_amount
    bne +
    inc temp_cast_amount
+
    bit flag_player_cpu
    bpl ++
    jmp ai_cast_raisedead
++  ; human
    ldy acoord
    lda arena_above,y
    bne +
-   rts
+   lda arena_dead,y
    and #DEAD_MASK_IS_DEAD
    beq -
    jsr is_spell_in_range   ; in range?
    bcs -
    lda wcoord
    ldx #0  ; check LOS
    jsr los_blocked
    bcc -
    ; got to here then spell is valid
    jsr do_raisedead_cast
    jsr cast_print_success_status
    rts


do_raisedead_cast:
    bit flag_player_cpu
    bpl +
    jsr set_spell_success
+   bit temp_success_flag
    bpl ++
+   ldy acoord
    lda #0
    sta arena_dead,y
    sta arena_bflag,y   ; just in case
    lda player_i
    ora #(AFLAG_MASK_UNDEAD | AFLAG_MASK_DISBELIEVE)    ; is real, is known to the ai to be real and is undead
    sta arena_aflag,y
++  jsr spell_animation
    lda #0
    sta temp_cast_amount
    rts


setup_wizard_spell:
    bit flag_player_cpu
    bpl ++
    jsr cast_print_spell_name
    lda #80
    jsr delay
    jsr set_spell_success
++
    lda wcoord
    sta acoord
    sta scoord
    jsr spell_animation
    lda #0
    sta temp_cast_amount
    rts

.cast_thing_mask_cpu = * - SPELL_MAGIC_SHIELD
!by PDATA_MASK_FLAGS1_SHIELD    ; shield
!by PDATA_MASK_FLAGS1_ARMOUR    ; armour
!by PDATA_MASK_FLAGS1_SWORD ; sword
!by PDATA_MASK_FLAGS1_KNIFE|PDATA_MASK_FLAGS1_SWORD ; knife
!by 0   ; (bow)
!by PDATA_MASK_FLAGS1_WINGS ; wings
!by 0,0,0,0 ; (law 1 .. chaos 2)
!by PDATA_MASK_FLAGS1_SHADOW    ; shadow form
.cast_thing_mask_and = * - SPELL_MAGIC_SHIELD
!by <!PDATA_MASK_FLAGS1_ARMOUR  ; shield
!by $ff ; armour
!by !(PDATA_MASK_FLAGS1_KNIFE | 1) ; sword
!by !(PDATA_MASK_FLAGS1_SWORD | PDATA_MASK_FLAGS1_KNIFE | 1) ; knife
!by 0   ; (bow)
!by $ff ; wings
!by 0,0,0,0 ; (law 1 .. chaos 2)
!by $ff ; shadow form
.cast_thing_mask_ora = * - SPELL_MAGIC_SHIELD
!by PDATA_MASK_FLAGS1_SHIELD    ; shield
!by PDATA_MASK_FLAGS1_ARMOUR|PDATA_MASK_FLAGS1_SHIELD    ; armour
!by PDATA_MASK_FLAGS1_SWORD ; sword
!by PDATA_MASK_FLAGS1_KNIFE ; knife
!by 0   ; (bow)
!by PDATA_MASK_FLAGS1_WINGS ; wings
!by 0,0,0,0 ; (law 1 .. chaos 2)
!by PDATA_MASK_FLAGS1_SHADOW    ; shadow form

cast_magic_thing:
    bit flag_player_cpu
    bpl ++
    ldx player_offs
    lda player_data + PDATA_OFFS_FLAGS1,x
    ldx current_spell
    and .cast_thing_mask_cpu,x
    beq +
    ; already have something similar
    lda #0
    sta target_square_found
    rts
+   lda #$80
    sta target_square_found
++  jsr do_thing_cast
    rts

do_thing_cast:
    jsr setup_wizard_spell
    bit temp_success_flag
    bpl +
    ldy player_offs
    ldx current_spell
    lda player_data + PDATA_OFFS_FLAGS1,y
    and .cast_thing_mask_and,x
    ora .cast_thing_mask_ora,x
    sta player_data + PDATA_OFFS_FLAGS1,y
+   jsr cast_print_success_status
    rts


cast_magic_bow:
    bit flag_player_cpu
    bpl ++
    ldx player_offs
    lda player_data + PDATA_OFFS_RANGED,x
    bne +
    ; already have a bow
    lda #0
    sta target_square_found
    rts
+   lda #$80
    sta target_square_found
++  jsr do_bow_cast
    rts

do_bow_cast:
    jsr setup_wizard_spell
    bit temp_success_flag
    bpl +
    ldy player_offs
    lda #$36    ; (ranged_combat << 4) | range
    sta player_data + PDATA_OFFS_RANGED,y
+   jsr cast_print_success_status
    rts


cast_disbelieve:
    lda temp_cast_amount
    bne +
    inc temp_cast_amount
+   bit flag_player_cpu
    bpl ++
    jsr ai_cast_disbelieve
    bit target_square_found
    bpl +
    jsr do_disbelieve_cast
+   rts
++  ; human
    ldy acoord
    lda arena_above,y
    beq +++
    cmp #SPELL_GOOEY_BLOB
    bcs +++
    jsr do_disbelieve_cast
    lda #0
    sta temp_cast_amount
+++ rts


do_disbelieve_cast:
    jsr spell_animation
    lda #0
    sta temp_success_flag
    ldy acoord
    lda arena_aflag,y
    and #AFLAG_MASK_ILLUSION
    beq ++
    ; place the arena_below riding wizard, or 0, in arena_above
    lda arena_below,y
    sta arena_above,y
    tax
    lda #0
    sta arena_below,y
    cpx #0  ; above
    beq +
    lda arena_bflag,y
    beq +
    ; what about dead bodies? but only if arena_below was empty
    sta arena_above,y
    lda arena_aflag,y
    ora #DEAD_MASK_IS_DEAD
    sta arena_dead,y
    lda #0
    sta arena_bflag,y
+   ldx #COLOR_YELLOW
    jsr pop_animation
    dec temp_success_flag
++  jsr cast_print_success_status
    lda #10
    jsr delay
    ; set AFLAG_MASK_DISBELIEVE for this creature (won't matter if disbelieved already...)
    ; the flag is "has been disbelieved" and is used for the AI to know which creatures
    ; are known to be real and which are not known...
    ldy acoord
    lda arena_aflag,y
    ora #AFLAG_MASK_DISBELIEVE
    sta arena_aflag,y
    rts


cast_turmoil:
    bit flag_player_cpu
    bpl ++
    jsr ai_cast_turmoil
    bit target_square_found
    bpl +
    jsr do_turmoil_cast
+   rts
++  ; humans can cast turmoil at their own risk!
    lda #1
    sta temp_cast_amount
    ; FALL THROUGH

do_turmoil_cast:
    ; according to chaos-0.82 turmoil always succeeds... ignoring that
!if DEBUG_MAGIC_ALWAYS_WORKS = 0 {
    bit temp_success_flag
    bmi +
    jmp cast_finish_print_success
}
+   ; XXX we could check if there are any empty squares here to avoid a lockup
    jsr unset_moved_flags
    ldy #-1
.do_turmoil_cast_loop:
    iny
    cpy #AREA_SIZE
    bne +
    ; done
    lda wcoord
    sta scoord
    lda #0
    sta temp_cast_amount
    jmp cast_finish_print_success
+   lda arena_above,y
    beq .do_turmoil_cast_loop
    lda arena_aflag,y
    and #AFLAG_MASK_MOVED
    bne .do_turmoil_cast_loop   ; already moved by turmoil
    ; get a new square to send this baby to
    lda #AREA_SIZE
    jsr get_random_a
-   tax
    lda arena_above,x
    beq ++
    ; not empty, check another square until found
    txa
    clc
    adc #COLS * 2 + 1   ; gcd(x, AREA_SIZE) == 1
    cmp #AREA_SIZE
    bcc -
    ; sec
    sbc #AREA_SIZE
    bcs -   ; bra
++  ; got a new square to cast to ; from y to x
    sty scoord
    stx acoord

    ; move the creature to the target square
    lda arena_above,y
    sta arena_above,x
    lda arena_aflag,y
    ora #AFLAG_MASK_MOVED   ; moved by turmoil
    sta arena_aflag,x
    lda arena_below,y
    sta arena_below,x
    lda arena_bflag,y
    sta arena_bflag,x
    lda arena_dead,y
    sta arena_dead,x
    lda #0
    sta arena_above,y
    sta arena_aflag,y
    sta arena_below,y
    sta arena_bflag,y
    sta arena_dead,y

    jsr spell_animation

    ldy scoord
    lda #4
    jsr delay
    jmp .do_turmoil_cast_loop


auto_cast_trees_castles:
    bit flag_player_cpu
    bpl +
    jsr set_spell_success
+
    lda wcoord
    ldx #$d
    jsr create_range_table
    sta lut_index

    ; loop over all spell
-   jsr get_best_index
    bcc .auto_cast_trees_castles_done
    ldx acoord
    lda arena_above,x
    bne -
    jsr is_tree_adjacent_tree_spell
    bcs -
    lda wcoord
    ldx #0  ; check LOS
    jsr los_blocked
    bcc -

    ; do spell cast anim, etc
    jsr spell_animation
    ; check success
    bit temp_success_flag
    bpl .auto_cast_trees_castles_done
    ; create tree/castle
    ldy acoord
    lda current_spell
    sta arena_above,y
    lda player_i
    sta arena_aflag,y
    jsr arena_draw_square
    lda #10
    jsr delay
    dec temp_cast_amount
    bne -

.auto_cast_trees_castles_done:
    ; assume for cpu that the spell was worth casting
    ; so cpu doesn't semi check for "line of sight" stuff when seeing if spell is worth casting
    lda #$80
    sta target_square_found
    lda #0
    sta temp_cast_amount
    jsr cast_print_success_status
    rts

} ; !zn Magic
