; phase 0 init, run once

!ifdef INCLUDE_PHASE0 !eof
!if INCLUDE_GUARD > 0 {
INCLUDE_PHASE0 = 1
}

!zn Phase0_Init {   ; all within the zone can be overwritten after run once

.SCREEN_X_P = 12
.SCREEN_Y_P = 47
.SCREEN_X_N = 5
.SCREEN_Y_N = 36
.RASTER_P = 131
.RASTER_N = 118

; # p0 code
init_phase0:
    sei
    ldx #<stack_top
    txs
    cld
    lda #$7f
    sta $913d
    sta $913e

    ; initialize VIC/VIA
    ldx #$f
-   lda .gamefw_tv_tbl_vic,x
    sta $9000,x
    dex
    bpl -
    lda #0
    ldx #$a
-   sta $9132,x
    dex
    bpl -

    ; setup screen
    ldx #0
-   lda #' '
    sta vmatrix,x
    sta vmatrix + COLS * ROWS - $100,x
    lda #1
    sta colorram,x
    sta colorram + COLS * ROWS - $100,x
    inx
    bne -

    lda #'#'
    ldx #AREA_W + 1
-   sta vmatrix,x
    sta vmatrix + COLS * (AREA_H + 1),x
    dex
    bpl -

    ldx #AREA_H - 1
    ldy #0
    clc
-   lda #'#'
    sta vmatrix + COLS * 1,y
    tya
    adc #AREA_W + 1
    tay
    lda #'#'
    sta vmatrix + COLS * 1,y
    tya
    adc #COLS - (AREA_W + 1)
    tay
    dex
    bpl -


    ldy #0
    ldx #0
    lda #6  ; lines
    sta zpdsth
--  lda #5  ; linelen
    sta zpdst
-   lda .text_stats,y
    sta vmatrix + POS_INFOLINE,x
    lda #0  ; black
    sta colorram + POS_INFOLINE,x
    iny
    inx
    dec zpdst
    bne -
    txa
    clc
    adc #COLS - 5
    tax
    dec zpdsth
    bne --

    ; init variables
    lda #NUM_PLAYERS_MIN
    sta player_num
    sta joy_count
    lda #0
    sta priority_offset
    sta sfx_enabled
    sta joy_prev
    sta joy_value

    ; detect pal/ntsc
    lda $ede4
    and #$01
    tax ; 0 -> pal, 1 -> ntsc
    beq +
    lda #.SCREEN_X_N
    sta $9000
    lda #.SCREEN_Y_N
    sta $9001
+

    lda #>vmatrix
    sta kv_hibase
    lda #$80
    sta kv_rptflg
    sta kv_mode

    ; enable Timer A free run on VIA 1
    lda #$40
    sta $912b

    lda #<irq_handler
    sta $0314
    lda #>irq_handler
    sta $0315
    lda #<brk_handler
    sta $0316
    lda #>brk_handler
    sta $0317
    lda #<nmi_handler
    sta $0318
    lda #>nmi_handler
    sta $0319

    lda .gamefw_tv_tbl_raster,x
    pha
    lda .gamefw_tv_tbl_timer_l,x
    tay
    lda .gamefw_tv_tbl_timer_h,x
    tax
    ; synchronise with the screen
    pla
-   cmp $9004
    beq -
-   cmp $9004
    bne -

    sty $9126
    stx $9125

    lda #$c0
    cli

!if * < player_names {
!error "player names init code under player names data"
!error player_names - *
}

    ; create initial player names
    ldx #PNAME_LEN - 1
-   lda .wizname,x
!for .i, 0,  NUM_PLAYERS_MAX - 1 {
    sta player_names + PNAME_LEN * .i,x
}
    dex
    bpl -

    ldx #NUM_PLAYERS_MAX - 1
-   txa
    asl
    asl
    asl
    tay
    txa
    ;clc
    adc #'1'
    sta player_names + .POS_WIZNAME_NUM,y
    dex
    bpl -

    ; create area->screen, area->x and area->y LUTs
    ldy #0
    sty zptmp   ; screen
!if FAST_GETYX > 0 {
    sty zpdsth  ; y
}
--  lda #0
    sta zpdst   ; x
    ldx #AREA_W
-   lda zptmp   ; screen
    sta tbl_area_i_to_screen,y
!if FAST_GETYX > 0 {
    lda zpdst   ; x
    sta tbl_area_i_x,y
    lda zpdsth  ; y
    sta tbl_area_i_y,y
}
    inc zptmp   ; screen
!if FAST_GETYX > 0 {
    inc zpdst   ; x
}
    iny
    dex
    bne -
!if FAST_GETYX > 0 {
    inc zpdsth  ; y
}
    lda zptmp   ; screen
    clc
    adc #COLS - AREA_W
    sta zptmp   ; screen
    cpy #AREA_SIZE
    bne --
    jmp main_entry

!if * > tbl_area_i_to_screen {
!error "tbl_area_i_to_screen init code under data"
!error * - tbl_area_i_to_screen
}

; # p0 data

.gamefw_tv_tbl_vic:
!by .SCREEN_X_P, .SCREEN_Y_P, 0, ROWS << 1, 0
!by ((((vmatrix & $1c00) >> 10) | ((vmatrix & $8000) >> 12) XOR 8) << 4) | (((charset & $1c00) >> 10) | ((charset & $8000) >> 12) XOR 8)
!by 0, 0, 0, 0
!by 0, 0, 0, 0
!by SOUND_VOLUME, 8 | COLOR_BLACK

; tbl: index 0 -> pal, 1 -> ntsc
.gamefw_tv_tbl_raster:
!by .RASTER_P, .RASTER_N
.gamefw_tv_tbl_timer_l:
!by <(312 * 71 - 2), <(261 * 65 - 2)
.gamefw_tv_tbl_timer_h:
!by >(312 * 71 - 2), >(261 * 65 - 2)

.text_stats:
!tx "COM=N"
!tx "R=N/N"
!tx "DEF=N"
!tx "MOV=N"
!tx "MAN=N"
!tx "MR=NN"

.wizname:
!tx "Wizard  "
!if (* - .wizname) != PNAME_LEN {
!error "wizname template has wrong len"
}
.POS_WIZNAME_NUM = 7

IP0_End = *
} ; !zn Phase0_Init
