; ### SFX test ###
; For 8k.

; $1000 - $11ff : vmatrix
; $1201 - $12xx : basic start
; $12xx - $1xxx : ml entry, main code
; $1xxx - $yyyy : sound data

; ###############

; # const

INCLUDE_GUARD = 0
DEBUG_IRQ_LEN = 1

COLS = 17
ROWS = 23

SOUND_VOLUME = 8

COLOR_BLACK = 0
COLOR_WHITE = 1
COLOR_RED = 2
COLOR_CYAN = 3
COLOR_PURPLE = 4
COLOR_GREEN = 5
COLOR_BLUE = 6
COLOR_YELLOW = 7

NORMAL_900F = (COLOR_WHITE << 4) | 8 | COLOR_CYAN

; ###############

; # addr

irq_downcount = $39
Sfx_zp_base = $3b

; $c5   ; used by keyscan
; $c6   ; used by keyscan
; $cb   ; used by keyscan
; $f5   ; used by keyscan
; $f6   ; used by keyscan

; used by keyscan etc
; $277
; $278
kv_hibase = $288
; $289
kv_rptflg = $28a
; $28c
; $28d
; $28e
; $28f
; $290
kv_mode = $291

vmatrix = $1000
charset = $8800
colorram = $9400 + (vmatrix & $200)

k_getinkbd = $f1f9 ;$ffe4


; ###############

!zn Basic_entry {
* = $1201
basic_start:
!wo basic_stop
!wo 0
!pet $9e, "4621", 0
basic_stop:
!wo 0
ml_entry:
    jmp init_phase0
} ; !zn Basic_entry

; ###############

!zn INT_Handler {
irq_handler:
    lda $9124   ; ACK IRQ
!if DEBUG_IRQ_LEN > 0 {
    lda #NORMAL_900F XOR $11
    sta $900f
}
    jsr sfx_tick

    ldy irq_downcount
    beq +
    dey
    sty irq_downcount
+
    ; read the keyboard
    jsr $ff9f

irq_exit:
; DEBUG
!if DEBUG_IRQ_LEN > 0 {
    lda #NORMAL_900F
    sta $900f
}

;brk_handler:
    pla
    tay
    pla
    tax
    pla

nmi_handler:
    rti

brk_handler:
!by 2   ; JAM

}   ; !zn INT_Handler


; ###############

; # init code

!zn Phase0_Init {

.SCREEN_X_P = 12
.SCREEN_Y_P = 38
.SCREEN_X_N = 5
.SCREEN_Y_N = 25
.RASTER_P = 131
.RASTER_N = 118

; # p0 data

.gamefw_tv_tbl_vic:
!by .SCREEN_X_P, .SCREEN_Y_P, (((vmatrix & $200) >> 2) | COLS), ROWS << 1, 0
!by ((((vmatrix & $1c00) >> 10) | ((vmatrix & $8000) >> 12) XOR 8) << 4) | (((charset & $1c00) >> 10) | ((charset & $8000) >> 12) XOR 8)
!by 0, 0, 0, 0
!by 0, 0, 0, 0
!by SOUND_VOLUME, NORMAL_900F

; tbl: index 0 -> pal, 1 -> ntsc
.gamefw_tv_tbl_raster:
!by .RASTER_P, .RASTER_N
.gamefw_tv_tbl_timer_l:
!by <(312 * 71 - 2), <(261 * 65 - 2)
.gamefw_tv_tbl_timer_h:
!by >(312 * 71 - 2), >(261 * 65 - 2)

.text_starttext:
;    01234567890123456
!tx "PRESS 0-9, A-F"
LEN_text_starttext = * - .text_starttext

; # p0 code

init_phase0:
    sei
    ldx #$e0    ; leave room to top of page 1
    txs
    cld
    lda #$7f
    sta $913d
    sta $913e

    ; initialize VIC/VIA
    ldx #$f
-   lda .gamefw_tv_tbl_vic,x
    sta $9000,x
    dex
    bpl -
    lda #0
    ldx #$a
-   sta $9132,x
    dex
    bpl -

    ; setup screen
    ldx #0
-   lda #' '
    sta vmatrix,x
    sta vmatrix + COLS * ROWS - $100,x
    lda #COLOR_BLUE
    sta colorram,x
    sta colorram + COLS * ROWS - $100,x
    inx
    bne -

    ldx #LEN_text_starttext
-   lda .text_starttext - 1,x
    sta vmatrix - 1,x
    dex
    bne -

    ; init variables
    jsr sfx_stop

    ; detect pal/ntsc
    lda $ede4
    and #$01
    tax ; 0 -> pal, 1 -> ntsc
    beq +
    lda #.SCREEN_X_N
    sta $9000
    lda #.SCREEN_Y_N
    sta $9001
+
    lda #>vmatrix
    sta kv_hibase
    lda #$80
    sta kv_rptflg
    sta kv_mode

    ; enable Timer A free run on VIA 1
    lda #$40
    sta $912b

    lda #<irq_handler
    sta $0314
    lda #>irq_handler
    sta $0315
    lda #<brk_handler
    sta $0316
    lda #>brk_handler
    sta $0317
    lda #<nmi_handler
    sta $0318
    lda #>nmi_handler
    sta $0319

    lda .gamefw_tv_tbl_raster,x
    pha
    lda .gamefw_tv_tbl_timer_l,x
    tay
    lda .gamefw_tv_tbl_timer_h,x
    tax
    ; synchronise with the screen
    pla
-   cmp $9004
    beq -
-   cmp $9004
    bne -

    sty $9126
    stx $9125

    lda #$c0
    cli
    jmp main_entry
} ; !zn Phase0_Ini


; ###############

; # main code

!zn Main_code {

; in:
; a = hex digit (ASCII)
; out:
; C = 0 if input ok
; a = value
hexkey_to_val:
    sec
    sbc #$30
    bcs +
--  sec
    rts
+   cmp #9 + 1
    bcs +
    rts
+   ; sec
    sbc #$41 - $30 - $a
    cmp #$a
    bcc --
    cmp #$f + 1
    rts

main_entry:
main_loop:
--  lda #1
    sta irq_downcount
-   lda irq_downcount
    bne -
    jsr k_getinkbd
    beq --
    jsr hexkey_to_val
    bcs --
    jsr sound_play
    jmp main_loop

} ; !zn Main_code


; ###############

Start_sfx = *
!src "sound.asm"
LEN_sfx = * - Start_sfx

